@@ -5,5 +5,5 @@
 */
 
 pub mod camera;
-pub mod text;
+//pub mod text;
 pub mod default_resources;
